<?php

namespace Drupal\vlsuite_block\Plugin\Block;

use Drupal\views\ViewExecutableFactory;
use Drupal\Core\Entity\EntityStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\vlsuite_utility_classes\VLSuiteUtilityClassesHelper;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\Block\ViewsBlock;
use Drupal\Component\Utility\Html;

/**
 * Defines the VLSuite block: list (views) plugin type.
 *
 * @Block(
 *   id = "vlsuite_block_views_block",
 *   admin_label = @Translation("VLSuite: Views block"),
 *   category = @Translation("VLSuite: List block (views)"),
 *   deriver = "Drupal\vlsuite_block\Plugin\Derivative\VLSuiteViewsBlockDeriver"
 * )
 *
 * @internal
 *   Plugin classes are internal.
 */
class VLSuiteViewsBlock extends ViewsBlock {

  /**
   * {@inheritdoc}
   * @param Drupal\vlsuite_utility_classes\VLSuiteUtilityClassesHelper $utility_classes_helper
   *  The utility classes helper.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ViewExecutableFactory $executable_factory, EntityStorageInterface $storage, AccountInterface $user, VLSuiteUtilityClassesHelper $utility_classes_helper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $executable_factory, $storage, $user);

    $this->utilityClassesHelper = $utility_classes_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('views.executable'),
      $container->get('entity_type.manager')->getStorage('view'),
      $container->get('current_user'),
      $container->get('vlsuite_utility_classes.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'grid_cols_widths' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['admin_label']['#access'] = FALSE;
    $form['label_display']['#type'] = 'hidden';
    $form['label_display']['#default_value'] = FALSE;
    $form['label']['#type'] = 'hidden';
    $form['label']['#default_value'] = $this->t('VLSuite List Block');
    $form['views_label_checkbox']['#default_value'] = FALSE;
    $form['views_label_checkbox']['#type'] = 'hidden';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $grid_cols_widths = $this->configuration['grid_cols_widths'] ?? NULL;
    if (!empty($grid_cols_widths)) {
      $style_plugin = $this->view->display_handler->getOption('style');
      if ($style_plugin['type'] !== 'html_list') {
        $style_plugin['type'] = 'html_list';
      }
      $style_plugin['options']['wrapper_class'] = NULL;
      $col_classes = $this->utilityClassesHelper->getColPercentageOptionClasses($grid_cols_widths);
      $style_plugin['options']['row_class'] = implode(' ', $col_classes);
      $style_plugin['options']['default_row_class'] = FALSE;
      $row_classes = $this->utilityClassesHelper->getRowClasses();
      $list_unstyled_classes = $this->utilityClassesHelper->getListUnstyledClasses();
      $style_plugin['options']['class'] = implode(' ', array_merge($row_classes, $list_unstyled_classes)); // 'row list-unstyled'
      $this->view->display_handler->setOption('style', $style_plugin);
    }
    $build = parent::build();
    $build['#attached']['library'][] = 'vlsuite_block/block';
    $build['#attributes']['class'][] = 'vlsuite-block';
    $build['#attributes']['class'][] = 'vlsuite-block__' . Html::cleanCssIdentifier($this->getDerivativeId());
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $form['grid_cols_widths'] = [
      '#type' => 'select',
      '#title' => $this->t('Grid column widths'),
      '#empty_option' => $this->t('Default'),
      '#default_value' => $this->configuration['grid_cols_widths'],
      '#options' => [
        '100' => '100%',
        '50' => '50%/50%',
        '33' => '33%/34%/33%',
        '25' => '25%/25%/25%/25%',
      ],
      '#description' => $this->t('Choose the grid column widths for this list, make sure @items_per_block matches this setting (e.g: 3 or 6 - 33%/33%/33%)', [
        '@items_per_block' => $this->t('Items per block')
      ]),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['grid_cols_widths'] = $form_state->getValue('grid_cols_widths', '');
    parent::blockSubmit($form, $form_state);
  }

}
