<?php

namespace Drupal\vlsuite_layout\Plugin\Layout;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\media\MediaInterface;
use Drupal\Core\Template\Attribute;
use Drupal\vlsuite_utility_classes\VLSuiteUtilityClassesHelper;

/**
 * VLSuite Layout base.
 */
abstract class VLSuiteLayoutBase extends LayoutDefault implements PluginFormInterface, ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The utility classes helper.
   *
   * @var \Drupal\vlsuite_utility_classes\VLSuiteUtilityClassesHelper
   */
  protected $utilityClassHelper;

  const UTILITY_KEY_GUTTER = 'gutter';
  const UTILITY_KEY_GUTTER_DEFAULT = '5';
  const UTILITY_KEY_VERTICAL_SPACING = 'padding-y';
  const UTILITY_KEY_VERTICAL_SPACING_DEFAULT = '5';
  const UTILITY_KEY_MEDIA_BG_OPACITY = 'media-bg-opacity';
  const UTILITY_KEY_MEDIA_BG_VARIANT = 'media-bg-variant';
  const APPLY_TO_SECTION = 'vlsuite_layout:section';
  const APPLY_TO_ROW = 'vlsuite_layout:row';
  const APPLY_TO_REGION_TOP = 'vlsuite_layout:region_top';
  const APPLY_TO_MAIN_REGIONS = 'vlsuite_layout:main_regions';
  const APPLY_TO_REGION_BOTTOM = 'vlsuite_layout:region_bottom';

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->configFactory = $container->get('config.factory');
    $instance->utilityClassHelper = $container->get('vlsuite_utility_classes.helper');
    return $instance;
  }

  /**
   * Get utility classes definitions.
   *
   * @return array
   *   Definitions.
   */
  public static function getUtilityClassesDefinitions() {
    $utility_classes_definitions = [];
    $utility_classes_definitions[self::APPLY_TO_SECTION] = [
      'admin_title' => t('VLSuite - Section (layout)'),
      'form_element_title' => t('Section'),
    ];
    $utility_classes_definitions[self::APPLY_TO_ROW] = [
      'admin_title' => t('VLSuite - All regions (layout)'),
      'form_element_title' => t('All regions'),
    ];
    $utility_classes_definitions[self::APPLY_TO_REGION_TOP] = [
      'admin_title' => t('VLSuite - Region top (layout)'),
      'form_element_title' => t('Region top'),
    ];
    $utility_classes_definitions[self::APPLY_TO_MAIN_REGIONS] = [
      'admin_title' => t('VLSuite - Main regions (layout)'),
      'form_element_title' => t('Main regions'),
    ];
    $utility_classes_definitions[self::APPLY_TO_REGION_BOTTOM] = [
      'admin_title' => t('VLSuite - Region bottom (layout)'),
      'form_element_title' => t('Region bottom'),
    ];
    return $utility_classes_definitions;
  }

  /**
   * Get width options.
   *
   * @return array
   *   Width options.
   */
  abstract protected function getWidthOptions();

  /**
   * Get default width.
   *
   * @return string
   *   Default width.
   */
  abstract protected function getDefaultWidth();

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['column_widths'] = $this->getDefaultWidth();
    $configuration['identifier'] = NULL;
    $configuration['edge_to_edge_bg'] = TRUE;
    $configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY] = [
      self::APPLY_TO_SECTION => [
        self::UTILITY_KEY_VERTICAL_SPACING => self::UTILITY_KEY_VERTICAL_SPACING_DEFAULT,
      ],
      self::APPLY_TO_ROW => [
        self::UTILITY_KEY_GUTTER => self::UTILITY_KEY_GUTTER_DEFAULT,
      ],
    ];
    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $width_options = $this->getWidthOptions();
    $form['column_widths'] = [
      '#type' => count($width_options) > 1 ? 'select' : 'value',
      '#title' => $this->t('Main column widths'),
      '#default_value' => $this->configuration['column_widths'],
      '#options' => $this->getWidthOptions(),
      '#description' => $this->t('Choose the main column widths for this layout.'),
    ];
    $form['edge_to_edge'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Edge to edge'),
      '#default_value' => $this->configuration['edge_to_edge'] ?? NULL,
    ];
    $form['edge_to_edge_bg'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Edge to edge background'),
      '#default_value' => $this->configuration['edge_to_edge_bg'] ?? NULL,
    ];
    $form['media_bg'] = [
      '#type' => 'media_library',
      '#allowed_bundles' => ['vlsuite_image', 'vlsuite_local_video'],
      '#title' => $this->t('Media background (Image / video)'),
      '#default_value' => $this->configuration['media_bg'] ?? NULL,
    ];
    $form[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY] = $this->utilityClassHelper->getUtilitiesApplyToListFormElement(self::getUtilityClassesDefinitions(), $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY] ?? []);
    $form['identifier'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Identifier'),
      '#description' => $this->t('Section identifier for anchor linking'),
      '#default_value' => $this->configuration['identifier'] ?? NULL,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['column_widths'] = $form_state->getValue('column_widths');
    $this->configuration['edge_to_edge'] = $form_state->getValue('edge_to_edge');
    $this->configuration['media_bg'] = $form_state->getValue('media_bg');
    $this->configuration['edge_to_edge_bg'] = $form_state->getValue('edge_to_edge_bg');
    $this->configuration['identifier'] = $form_state->getValue('identifier');
    $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY] = $form_state->getValue(VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY, []);
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    if (empty($build['#attributes']) || !$build['#attributes'] instanceof Attribute) {
      $build['#attributes'] = new Attribute($build['#attributes'] ?? []);
    }
    $build['#attributes']->addClass('vlsuite-layout');
    $build['#attributes']->addClass(Html::cleanCssIdentifier($this->getPluginDefinition()->id()));
    $identifier = $this->configuration['identifier'];
    if (!empty($identifier)) {
      $build['#attributes']->setAttribute('id', $identifier);
    }
    $edge_to_edge = $this->configuration['edge_to_edge'] ?? FALSE;
    if (!$edge_to_edge) {
      $build['#attributes']->addClass($this->utilityClassHelper->getContainerClasses());
    }

    $section_utility_clases_config = $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY][self::APPLY_TO_SECTION] ?? [];
    $media_bg_opacity_classes = [];
    $media_bg_variant_classes = [];

    if (!empty($section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_OPACITY])) {
      $media_bg_opacity_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_SECTION, [
        self::UTILITY_KEY_MEDIA_BG_OPACITY => $section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_OPACITY],
      ]);
      unset($section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_OPACITY]);
    }
    if (!empty($section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_VARIANT])) {
      $media_bg_variant_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_SECTION, [
        self::UTILITY_KEY_MEDIA_BG_VARIANT => $section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_VARIANT],
      ]);
      unset($section_utility_clases_config[self::UTILITY_KEY_MEDIA_BG_VARIANT]);
    }
    $section_utility_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_SECTION, $section_utility_clases_config);
    $section_classes = array_merge(['vlsuite-layout-bg-wrapper'], $section_utility_classes);

    $edge_to_edge_bg = $this->configuration['edge_to_edge_bg'] ?? NULL;
    if (!empty($edge_to_edge_bg)) {
      $build['#theme_wrappers']['container']['#attributes']['class'] = array_merge($build['#theme_wrappers']['container']['#attributes']['class'] ?? [], $section_classes);
    }
    else {
      $build['#attributes']->addClass($section_classes);
    }
    $bg_media = $this->configuration['media_bg'] ?? NULL;

    if (!empty($bg_media)) {
      $look_up_for_media = $this->entityTypeManager->getStorage('media')->load($bg_media);
      if ($look_up_for_media instanceof MediaInterface) {
        $build['media_bg'] = ['#type' => 'container'];
        $build['media_bg']['media'] = $this->entityTypeManager->getViewBuilder('media')->view($look_up_for_media, 'vlsuite_background');
        $build['media_bg']['#attributes']['class'][] = 'vlsuite-layout-bg-wrapper__bg-media';
        if (!empty($media_bg_opacity_classes)) {
          $build['media_bg']['#attributes']['class'] = array_merge($media_bg_opacity_classes, $build['media_bg']['#attributes']['class']);
        }
        if (!empty($media_bg_variant_classes)) {
          $build['media_bg']['#attributes']['class'] = array_merge($media_bg_variant_classes, $build['media_bg']['#attributes']['class']);
        }
      }
    }

    $build['#row_attributes'] = new Attribute();
    $build['#row_attributes']->addClass($this->utilityClassHelper->getRowClasses());
    $row_utility_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_ROW, $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY][self::APPLY_TO_ROW] ?? []);
    $build['#row_attributes']->addClass($row_utility_classes);

    $top_utility_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_REGION_TOP, $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY][self::APPLY_TO_REGION_TOP] ?? []);
    $col_100_classes = $this->utilityClassHelper->getColPercentageOptionClasses('100');
    $top_classes = $top_utility_classes;

    $main_utility_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(static::APPLY_TO_MAIN_REGIONS, $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY][self::APPLY_TO_MAIN_REGIONS] ?? []);
    $main_classes = $main_utility_classes;

    $bottom_utility_classes = $this->utilityClassHelper->getUtilitiesApplyToClasses(self::APPLY_TO_REGION_BOTTOM, $this->configuration[VLSuiteUtilityClassesHelper::UTILITY_CLASSES_KEY][self::APPLY_TO_REGION_BOTTOM] ?? []);
    $bottom_classes = $bottom_utility_classes;

    $column_widths = $this->configuration['column_widths'];
    $column_widths_items = explode('-', $column_widths);
    $auto = in_array('auto', $column_widths_items);
    $auto_classes = $this->utilityClassHelper->getColPercentageOptionClasses('auto');
    foreach ($this->getPluginDefinition()->getRegionNames() as $region_name) {
      if ($region_name == 'top') {
        $build['#' . $region_name . '_col_attributes'] = new Attribute(['class' => $col_100_classes]);
        $build['#' . $region_name . '_class'] = $top_classes;
      }
      elseif ($region_name == 'bottom') {
        $build['#' . $region_name . '_col_attributes'] = new Attribute(['class' => $col_100_classes]);
        $build['#' . $region_name . '_class'] = $bottom_classes;
      }
      else {
        $build['#' . $region_name . '_col_attributes'] = new Attribute([
          'class' => $this->utilityClassHelper->getColPercentageOptionClasses(array_shift($column_widths_items)),
        ]);
        $build['#' . $region_name . '_col_attributes']->addClass($auto ? $auto_classes : []);
        $build['#' . $region_name . '_class'] = $main_classes;
        // Make sure sections initialized when empty to avoid grid to fail when
        // using empty regions combined with background image for example.
        if (empty($build[$region_name])) {
          $build[$region_name] = [];
        }
      }
    }
    $build['#attached']['library'][] = 'vlsuite_layout/layout';
    return $build;
  }

}
