<?php

namespace Drupal\vlsuite_utility_classes;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Helper "VLSuiteUtilityClassesHelper" service object.
 */
class VLSuiteUtilityClassesHelper {

  use StringTranslationTrait;

  const UTILITY_CLASSES_KEY = 'vlsuite_utility_class';

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The module handler object.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Utility apply to options.
   *
   * @var array
   */
  protected $utilityApplyToOptions;

  /**
   * Constructs a "VLSuiteUtilityClassesHelper" object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A configuration factory instance.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   A module handler instance.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ModuleHandlerInterface $module_handler) {
    $this->configFactory = $config_factory;
    $this->moduleHandler = $module_handler;
  }

  /**
   * Get utility apply to options.
   *
   * @return array
   *   Options.
   */
  public function getUtilityApplyToOptions() {
    if (empty($this->utilityApplyToOptions)) {
      $apply_to_options = [];
      $this->moduleHandler->alter('vlsuite_utility_classes_utility_apply_to_options', $apply_to_options);
      $this->utilityApplyToOptions = $apply_to_options;
    }
    return $this->utilityApplyToOptions;
  }

  /**
   * Get utility apply to list of utilities.
   *
   * @param string $apply_to
   *   Apply to.
   *
   * @return array
   *   Utilities for passed apply to.
   */
  public function getUtilitiesApplyToList($apply_to) {
    $utilities = $this->configFactory->get('vlsuite_utility_classes.settings')->get('utilities');
    $utilities_apply_to = [];
    foreach ($utilities as $utility_identifier => $utility) {
      if (!empty($utility['apply_to'][$apply_to])) {
        unset($utility['apply_to']);
        $utilities_apply_to[$utility_identifier] = $utility;
      }
    }
    return $utilities_apply_to;
  }

  /**
   * Get utility key value classes.
   *
   * @param string $utility_key
   *   Utility key.
   * @param string $value
   *   Utility value.
   *
   * @return array
   *   Utility classes for passed value.
   */
  public function getUtilityKeyValueClasses($utility_key, $value) {
    $utilities = $this->configFactory->get('vlsuite_utility_classes.settings')->get('utilities');
    $class_prefix = $utilities[$utility_key]['class_prefix'] ?? '';
    $class_value = isset($utilities[$utility_key]['values'][$value]) ? $value : '';
    // @todo utility classes with value "0" not added, like gutter for
    // section row.
    // Defined by prefix.
    if ($class_value == 'defined-by-prefix') {
      $class_value = '';
    }
    $class_mix = $class_prefix . $class_value;
    return array_filter(explode(' ', $class_mix));
  }

  /**
   * Get utilities apply to classes.
   *
   * @param string $apply_to
   *   Apply to.
   * @param array $utility_classes_config
   *   Utility classes config.
   *
   * @return array
   *   Utilities apply to classes.
   */
  public function getUtilitiesApplyToClasses($apply_to, array $utility_classes_config) {
    $apply_to_utilities = $this->getUtilitiesApplyToList($apply_to);
    $apply_to_config = $utility_classes_config ?? [];
    $apply_to_classes = [];
    foreach (array_keys($apply_to_utilities) as $utility_key) {
      if (!empty($apply_to_config[$utility_key])) {
        $utility_classes = $this->getUtilityKeyValueClasses($utility_key, $apply_to_config[$utility_key]);
        $apply_to_classes = array_merge($apply_to_classes, $utility_classes);
      }
    }
    return $apply_to_classes;
  }

  /**
   * Get container classes.
   *
   * @return array
   *   Classes.
   */
  public function getContainerClasses() {
    $container_classes = $this->configFactory->get('vlsuite_utility_classes.settings')->get('container_classes') ?? '';
    return explode(' ', $container_classes);
  }

  /**
   * Get list unstyled classes.
   *
   * @return array
   *   Classes.
   */
  public function getListUnstyledClasses() {
    $list_unstyled_classes = $this->configFactory->get('vlsuite_utility_classes.settings')->get('list_unstyled_classes') ?? '';
    return explode(' ', $list_unstyled_classes);
  }

  /**
   * Get row classes.
   *
   * @return array
   *   Classes.
   */
  public function getRowClasses() {
    $container_classes = $this->configFactory->get('vlsuite_utility_classes.settings')->get('row_classes') ?? '';
    return explode(' ', $container_classes);
  }

  /**
   * Get col variant keys.
   *
   * @return array
   *   List.
   */
  public static function getColPercentageOptions() {
    return ['100', '80', '75', '67', '60', '50', '40', '33', '25'];
  }

  /**
   * Get col option classes.
   *
   * @param string $col_option
   *   Option (percentage or auto).
   *
   * @return array
   *   Classes.
   */
  public function getColPercentageOptionClasses($col_option) {
    $col_classes = $this->configFactory->get('vlsuite_utility_classes.settings')->get('col_classes')['col_' . $col_option] ?? '';
    return explode(' ', $col_classes);
  }

  /**
   * Get utilities apply to list form element.
   *
   * @param array $apply_to_definitions
   *   Apply to definitions.
   * @param array $defaults
   *   Config for default values.
   *
   * @return array
   *   Form element.
   */
  public function getUtilitiesApplyToListFormElement(array $apply_to_definitions, array $defaults) {
    $element = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#attached' => ['library' => ['vlsuite_utility_classes/previewer']],
      '#attributes' => ['class' => ['vlsuite-utility-classes']],
    ];
    $element['previewer'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['vlsuite-utility-classes__previewer']],
      'text' => ['#markup' => $this->t('New selection previewer')],
    ];
    $has_any = FALSE;
    foreach ($apply_to_definitions as $utility_classes_apply_to_key => $definition) {
      $utilities = $this->getUtilitiesApplyToList($utility_classes_apply_to_key);
      $element[$utility_classes_apply_to_key] = [
        '#type' => 'details',
        '#title' => $definition['form_element_title'],
        '#open' => FALSE,
        '#access' => !empty($utilities),
      ];
      $has_any = $has_any ? $has_any : !empty($utilities);
      foreach ($utilities as $utility_identifier => $utility) {
        $element[$utility_classes_apply_to_key][$utility_identifier] = [
          '#type' => 'select',
          '#empty_option' => $this->t('Default'),
          '#options' => $utility['values'],
          '#title' => $utility['visual_name'],
          '#default_value' => $defaults[$utility_classes_apply_to_key][$utility_identifier] ?? NULL,
          '#attributes' => [
            'class' => ['vlsuite-utility-classes__previewer-option'],
            'data-class-prefix' => $utility['class_prefix'],
            'data-class-key' => $utility_identifier,
          ],
        ];
      }
    }
    if (!$has_any) {
      $element['#access'] = FALSE;
    }
    return $element;
  }

  /**
   * Helper to apply utility classes to build array parents (mainly for block).
   *
   * @param array $build
   *   Build.
   * @param array $classes
   *   Classes array.
   * @param array $parents_group
   *   Parents of element where to apply classes.
   */
  protected function buildApplyUtilityClassParents(array &$build, $classes, array $parents_group) {
    foreach ($parents_group as $parents) {
    $ref = &$build;
      foreach (explode(':', $parents) as $parent) {
        $new_restriction = (str_contains($parent, 'attributes') || str_contains($parent, 'class'));
        if (is_array($ref)) {
          if (!isset($ref[$parent]) && (str_contains($parent, 'attributes') || str_contains($parent, 'class'))) {
            $ref[$parent] = [];
          }
          $ref = &$ref[$parent];

        }
        elseif (is_object($ref)) {
          if (!isset($ref->$parent) && (str_contains($parent, 'attributes') || str_contains($parent, 'class'))) {
            $ref->$parent = [];
          }
          $ref = &$ref->$parent;
        }
      }
      if ($ref !== NULL) {
        $ref = array_merge($ref, $classes);
      }
    }
  }

  /**
   * Helper to apply utility classes to build array (mainly for block).
   *
   * @param array $apply_to_list
   *   Apply to list.
   * @param array $build
   *   Build.
   */
  public function buildApplyUtilityClasses(array $apply_to_list, array &$build) {
    $is_field = !empty($build['#theme']) && $build['#theme'] == 'field';
    foreach ($apply_to_list as $apply_to => $utility_classes_config) {
      $classes = $this->getUtilitiesApplyToClasses($apply_to, $utility_classes_config) ?? NULL;
      [, $bundle, $field, $item] = explode(':', $apply_to);
      // @TODO Cardinalliy according to items!
      $parents_group = [];
      if (!empty($item)) {
        // @see template_preprocess_field().
        $parents_group[] = (!$is_field ? $field . ':' : '') . '0:#attributes:class';
        // @code $parents_group[] = (!$is_field ? $field . ':' : '') . '0:_attributes:class'; @endcode
        $parents_group[] = (!$is_field ? $field . ':' : '') . '0:#item_attributes:class';
      }
      elseif (!empty($field)) {
        $parents_group[] = (!$is_field ? $field . ':' : '') . '#attributes:class';
      }
      elseif (!empty($bundle)) {
        $parents_group[] = '#attributes:class';
      }
      if (!empty($classes) && !empty($parents_group)) {
        $this->buildApplyUtilityClassParents($build, $classes, $parents_group);
      }
    }
  }

}
